@@ -8,7 +8,7 @@ Contact at www.sinclair.bio
 """
 
 # Built-in modules #
-import os, shutil
+import os, sys, shutil
 
 # Internal modules #
 from fasta import FASTA
@@ -37,8 +37,15 @@ class FASTQ(FASTA):
     #----------------------------- Properties --------------------------------#
     @property_cached
     def count(self):
-        if self.gzipped: return int(sh.zgrep('-c', "^+$", self.path, _ok_code=[0,1]))
-        return int(sh.grep('-c', "^+$", self.path, _ok_code=[0,1]))
+        # Import module #
+        from shell_command import shell_output
+        # Case when we are not compressed #
+        if not self.gzipped:
+            return int(int(shell_output("cat %s | wc -l" % self.path)) / 4)
+        # If we are gzipped we can just use zcat or gzcat on macOS #
+        program = 'gzcat' if sys.platform != 'linux' else 'zcat'
+        command = "%s %s | wc -l" % (program, self.path)
+        return int(int(shell_output(command)) / 4)
 
     @property_cached
     def avg_quality(self):
